@@ -1,5 +1,3 @@
-from __future__ import print_function, division, absolute_import
-from __future__ import unicode_literals
 from fontTools.misc.testTools import getXML
 from fontTools.otlLib import builder
 from fontTools.ttLib.tables import otTables
@@ -472,7 +470,7 @@ class BuilderTest(object):
         assert getXML(lookup.toXML) == [
             "<Lookup>",
             '  <LookupType value="1"/>',
-            '  <LookupFlag value="7"/>',
+            '  <LookupFlag value="7"/><!-- rightToLeft ignoreBaseGlyphs ignoreLigatures -->',
             "  <!-- SubTableCount=2 -->",
             '  <SingleSubst index="0">',
             '    <Substitution in="one" out="two"/>',
@@ -526,7 +524,7 @@ class BuilderTest(object):
         assert getXML(lookup.toXML) == [
             "<Lookup>",
             '  <LookupType value="1"/>',
-            '  <LookupFlag value="17"/>',
+            '  <LookupFlag value="17"/><!-- rightToLeft useMarkFilteringSet -->',
             "  <!-- SubTableCount=1 -->",
             '  <SingleSubst index="0">',
             '    <Substitution in="one" out="two"/>',
