@@ -1,5 +1,3 @@
-from __future__ import \
-	print_function, division, absolute_import, unicode_literals
 from fontTools.misc.py23 import *
 from fontTools.misc.loggingTools import CapturingLogHandler
 from fontTools.misc.testTools import parseXML
@@ -18,9 +16,9 @@ def hexencode(s):
 
 
 AXES = {
-	"wdth": (0.3, 0.4, 0.5),
+	"wdth": (0.25, 0.375, 0.5),
 	"wght": (0.0, 1.0, 1.0),
-	"opsz": (-0.7, -0.7, 0.0)
+	"opsz": (-0.75, -0.75, 0.0)
 }
 
 
@@ -69,6 +67,13 @@ SKIA_GVAR_I_DATA = deHexStr(
 
 
 class TupleVariationTest(unittest.TestCase):
+	def __init__(self, methodName):
+		unittest.TestCase.__init__(self, methodName)
+		# Python 3 renamed assertRaisesRegexp to assertRaisesRegex,
+		# and fires deprecation warnings if a program uses the old name.
+		if not hasattr(self, "assertRaisesRegex"):
+			self.assertRaisesRegex = self.assertRaisesRegexp
+
 	def test_equal(self):
 		var1 = TupleVariation({"wght":(0.0, 1.0, 1.0)}, [(0,0), (9,8), (7,6)])
 		var2 = TupleVariation({"wght":(0.0, 1.0, 1.0)}, [(0,0), (9,8), (7,6)])
@@ -107,7 +112,7 @@ class TupleVariationTest(unittest.TestCase):
 		self.assertIn("bad delta format", [r.msg for r in captor.records])
 		self.assertEqual([
 			'<tuple>',
-			  '<coord axis="wdth" max="0.5" min="0.3" value="0.4"/>',
+			  '<coord axis="wdth" min="0.25" value="0.375" max="0.5"/>',
 			  '<!-- bad delta #0 -->',
 			'</tuple>',
 		], TupleVariationTest.xml_lines(writer))
@@ -118,9 +123,9 @@ class TupleVariationTest(unittest.TestCase):
 		g.toXML(writer, ["wdth", "wght", "opsz"])
 		self.assertEqual([
 			'<tuple>',
-			  '<coord axis="wdth" max="0.5" min="0.3" value="0.4"/>',
+			  '<coord axis="wdth" min="0.25" value="0.375" max="0.5"/>',
 			  '<coord axis="wght" value="1.0"/>',
-			  '<coord axis="opsz" value="-0.7"/>',
+			  '<coord axis="opsz" value="-0.75"/>',
 			  '<delta cvt="0" value="42"/>',
 			  '<delta cvt="2" value="23"/>',
 			  '<delta cvt="3" value="0"/>',
@@ -134,9 +139,9 @@ class TupleVariationTest(unittest.TestCase):
 		g.toXML(writer, ["wdth", "wght", "opsz"])
 		self.assertEqual([
 			'<tuple>',
-			  '<coord axis="wdth" max="0.5" min="0.3" value="0.4"/>',
+			  '<coord axis="wdth" min="0.25" value="0.375" max="0.5"/>',
 			  '<coord axis="wght" value="1.0"/>',
-			  '<coord axis="opsz" value="-0.7"/>',
+			  '<coord axis="opsz" value="-0.75"/>',
 			  '<delta pt="0" x="9" y="8"/>',
 			  '<delta pt="2" x="7" y="6"/>',
 			  '<delta pt="3" x="0" y="0"/>',
@@ -156,6 +161,22 @@ class TupleVariationTest(unittest.TestCase):
 			'</tuple>'
 		], TupleVariationTest.xml_lines(writer))
 
+	def test_toXML_axes_floats(self):
+		writer = XMLWriter(BytesIO())
+		axes = {
+			"wght": (0.0, 0.2999878, 0.7000122),
+			"wdth": (0.0, 0.4000244, 0.4000244),
+		}
+		g = TupleVariation(axes, [None] * 5)
+		g.toXML(writer, ["wght", "wdth"])
+		self.assertEqual(
+			[
+				'<coord axis="wght" min="0.0" value="0.3" max="0.7"/>',
+				'<coord axis="wdth" value="0.4"/>',
+			],
+			TupleVariationTest.xml_lines(writer)[1:3]
+		)
+
 	def test_fromXML_badDeltaFormat(self):
 		g = TupleVariation({}, [])
 		with CapturingLogHandler(log, "WARNING") as captor:
@@ -167,9 +188,9 @@ class TupleVariationTest(unittest.TestCase):
 	def test_fromXML_constants(self):
 		g = TupleVariation({}, [None] * 4)
 		for name, attrs, content in parseXML(
-				'<coord axis="wdth" min="0.3" value="0.4" max="0.5"/>'
+				'<coord axis="wdth" min="0.25" value="0.375" max="0.5"/>'
 				'<coord axis="wght" value="1.0"/>'
-				'<coord axis="opsz" value="-0.7"/>'
+				'<coord axis="opsz" value="-0.75"/>'
 				'<delta cvt="1" value="42"/>'
 				'<delta cvt="2" value="-23"/>'):
 			g.fromXML(name, attrs, content)
@@ -179,15 +200,31 @@ class TupleVariationTest(unittest.TestCase):
 	def test_fromXML_points(self):
 		g = TupleVariation({}, [None] * 4)
 		for name, attrs, content in parseXML(
-				'<coord axis="wdth" min="0.3" value="0.4" max="0.5"/>'
+				'<coord axis="wdth" min="0.25" value="0.375" max="0.5"/>'
 				'<coord axis="wght" value="1.0"/>'
-				'<coord axis="opsz" value="-0.7"/>'
+				'<coord axis="opsz" value="-0.75"/>'
 				'<delta pt="1" x="33" y="44"/>'
 				'<delta pt="2" x="-2" y="170"/>'):
 			g.fromXML(name, attrs, content)
 		self.assertEqual(AXES, g.axes)
 		self.assertEqual([None, (33, 44), (-2, 170), None], g.coordinates)
 
+	def test_fromXML_axes_floats(self):
+		g = TupleVariation({}, [None] * 4)
+		for name, attrs, content in parseXML(
+			'<coord axis="wght" min="0.0" value="0.3" max="0.7"/>'
+			'<coord axis="wdth" value="0.4"/>'
+		):
+			g.fromXML(name, attrs, content)
+
+		self.assertEqual(g.axes["wght"][0], 0)
+		self.assertAlmostEqual(g.axes["wght"][1], 0.2999878)
+		self.assertAlmostEqual(g.axes["wght"][2], 0.7000122)
+
+		self.assertEqual(g.axes["wdth"][0], 0)
+		self.assertAlmostEqual(g.axes["wdth"][1], 0.4000244)
+		self.assertAlmostEqual(g.axes["wdth"][2], 0.4000244)
+
 	def test_compile_sharedPeaks_nonIntermediate_sharedPoints(self):
 		var = TupleVariation(
 			{"wght": (0.0, 0.5, 0.5), "wdth": (0.0, 0.8, 0.8)},
@@ -370,7 +407,7 @@ class TupleVariationTest(unittest.TestCase):
 		self.assertEqual(({"wght": -1.0, "wdth": 0.5}, 6), decompileCoord(["wght", "wdth"], data, 2))
 
 	def test_decompileCoord_roundTrip(self):
-		# Make sure we are not affected by https://github.com/behdad/fonttools/issues/286
+		# Make sure we are not affected by https://github.com/fonttools/fonttools/issues/286
 		data = deHexStr("7F B9 80 35")
 		values, _ = TupleVariation.decompileCoord_(["wght", "wdth"], data, 0)
 		axisValues = {axis:(val, val, val) for axis, val in  values.items()}
@@ -681,6 +718,167 @@ class TupleVariationTest(unittest.TestCase):
 		content = writer.file.getvalue().decode("utf-8")
 		return [line.strip() for line in content.splitlines()][1:]
 
+	def test_getCoordWidth(self):
+		empty = TupleVariation({}, [])
+		self.assertEqual(empty.getCoordWidth(), 0)
+
+		empty = TupleVariation({}, [None])
+		self.assertEqual(empty.getCoordWidth(), 0)
+
+		gvarTuple = TupleVariation({}, [None, (0, 0)])
+		self.assertEqual(gvarTuple.getCoordWidth(), 2)
+
+		cvarTuple = TupleVariation({}, [None, 0])
+		self.assertEqual(cvarTuple.getCoordWidth(), 1)
+
+		cvarTuple.coordinates[1] *= 1.0
+		self.assertEqual(cvarTuple.getCoordWidth(), 1)
+
+		with self.assertRaises(TypeError):
+			TupleVariation({}, [None, "a"]).getCoordWidth()
+
+	def test_scaleDeltas_cvar(self):
+		var = TupleVariation({}, [100, None])
+
+		var.scaleDeltas(1.0)
+		self.assertEqual(var.coordinates, [100, None])
+
+		var.scaleDeltas(0.333)
+		self.assertAlmostEqual(var.coordinates[0], 33.3)
+		self.assertIsNone(var.coordinates[1])
+
+		var.scaleDeltas(0.0)
+		self.assertEqual(var.coordinates, [0, None])
+
+	def test_scaleDeltas_gvar(self):
+		var = TupleVariation({}, [(100, 200), None])
+
+		var.scaleDeltas(1.0)
+		self.assertEqual(var.coordinates, [(100, 200), None])
+
+		var.scaleDeltas(0.333)
+		self.assertAlmostEqual(var.coordinates[0][0], 33.3)
+		self.assertAlmostEqual(var.coordinates[0][1], 66.6)
+		self.assertIsNone(var.coordinates[1])
+
+		var.scaleDeltas(0.0)
+		self.assertEqual(var.coordinates, [(0, 0), None])
+
+	def test_roundDeltas_cvar(self):
+		var = TupleVariation({}, [55.5, None, 99.9])
+		var.roundDeltas()
+		self.assertEqual(var.coordinates, [56, None, 100])
+
+	def test_roundDeltas_gvar(self):
+		var = TupleVariation({}, [(55.5, 100.0), None, (99.9, 100.0)])
+		var.roundDeltas()
+		self.assertEqual(var.coordinates, [(56, 100), None, (100, 100)])
+
+	def test_calcInferredDeltas(self):
+		var = TupleVariation({}, [(0, 0), None, None, None])
+		coords = [(1, 1), (1, 1), (1, 1), (1, 1)]
+
+		var.calcInferredDeltas(coords, [])
+
+		self.assertEqual(
+			var.coordinates,
+			[(0, 0), (0, 0), (0, 0), (0, 0)]
+		)
+
+	def test_calcInferredDeltas_invalid(self):
+		# cvar tuples can't have inferred deltas
+		with self.assertRaises(TypeError):
+			TupleVariation({}, [0]).calcInferredDeltas([], [])
+
+		# origCoords must have same length as self.coordinates
+		with self.assertRaises(ValueError):
+			TupleVariation({}, [(0, 0), None]).calcInferredDeltas([], [])
+
+		# at least 4 phantom points required
+		with self.assertRaises(AssertionError):
+			TupleVariation({}, [(0, 0), None]).calcInferredDeltas([(0, 0), (0, 0)], [])
+
+		with self.assertRaises(AssertionError):
+			TupleVariation({}, [(0, 0)] + [None]*5).calcInferredDeltas(
+				[(0, 0)]*6,
+				[1, 0]  # endPts not in increasing order
+			)
+
+	def test_optimize(self):
+		var = TupleVariation({"wght": (0.0, 1.0, 1.0)}, [(0, 0)]*5)
+
+		var.optimize([(0, 0)]*5, [0])
+
+		self.assertEqual(var.coordinates, [None, None, None, None, None])
+
+	def test_optimize_isComposite(self):
+		# when a composite glyph's deltas are all (0, 0), we still want
+		# to write out an entry in gvar, else macOS doesn't apply any
+		# variations to the composite glyph (even if its individual components
+		# do vary).
+		# https://github.com/fonttools/fonttools/issues/1381
+		var = TupleVariation({"wght": (0.0, 1.0, 1.0)}, [(0, 0)]*5)
+		var.optimize([(0, 0)]*5, [0], isComposite=True)
+		self.assertEqual(var.coordinates, [(0, 0)]*5)
+
+		# it takes more than 128 (0, 0) deltas before the optimized tuple with
+		# (None) inferred deltas (except for the first) becomes smaller than
+		# the un-optimized one that has all deltas explicitly set to (0, 0).
+		var = TupleVariation({"wght": (0.0, 1.0, 1.0)}, [(0, 0)]*129)
+		var.optimize([(0, 0)]*129, list(range(129-4)), isComposite=True)
+		self.assertEqual(var.coordinates, [(0, 0)] + [None]*128)
+
+	def test_sum_deltas_gvar(self):
+		var1 = TupleVariation(
+			{},
+			[
+				(-20, 0), (-20, 0), (20, 0), (20, 0),
+				(0, 0), (0, 0), (0, 0), (0, 0),
+			]
+		)
+		var2 = TupleVariation(
+			{},
+			[
+				(-10, 0), (-10, 0), (10, 0), (10, 0),
+				(0, 0), (20, 0), (0, 0), (0, 0),
+			]
+		)
+
+		var1 += var2
+
+		self.assertEqual(
+			var1.coordinates,
+			[
+				(-30, 0), (-30, 0), (30, 0), (30, 0),
+				(0, 0), (20, 0), (0, 0), (0, 0),
+			]
+		)
+
+	def test_sum_deltas_gvar_invalid_length(self):
+		var1 = TupleVariation({}, [(1, 2)])
+		var2 = TupleVariation({}, [(1, 2), (3, 4)])
+
+		with self.assertRaisesRegex(ValueError, "deltas with different lengths"):
+			var1 += var2
+
+	def test_sum_deltas_gvar_with_inferred_points(self):
+		var1 = TupleVariation({}, [(1, 2), None])
+		var2 = TupleVariation({}, [(2, 3), None])
+
+		with self.assertRaisesRegex(ValueError, "deltas with inferred points"):
+			var1 += var2
+
+	def test_sum_deltas_cvar(self):
+		axes = {"wght": (0.0, 1.0, 1.0)}
+		var1 = TupleVariation(axes, [0, 1, None, None])
+		var2 = TupleVariation(axes, [None, 2, None, 3])
+		var3 = TupleVariation(axes, [None, None, None, 4])
+
+		var1 += var2
+		var1 += var3
+
+		self.assertEqual(var1.coordinates, [0, 3, None, 7])
+
 
 if __name__ == "__main__":
 	import sys
